//Test Vector Verification in Verilog
module testbench;


parameter strobe = 2;
parameter half_cycle = 10;
parameter vectors = 100;
integer index_i,index_o, index_b;
integer vi_F, vo_F, vb_F;

wire [7:0] SRAM_DATA;
reg [7:0] test;
reg [6:0] WR_addr;
wire [6:0] RD_addr;
reg  WR_EN;
reg  RD_EN;
reg  Aclr;
reg  WClock;
reg  RClock;

reg [11:0] iv ;
reg [6:0]  ov ;
reg [7:0]  bi ;

wire [6:0] RD_ADDR_exp;


reg [11:0] vector_i [vectors:1];
reg [6:0] vector_o [vectors:1];
reg [7:0] vector_b [vectors:1];


assign RD_ADDR_exp = ov;

assign SRAM_DATA = test;

task apply_iv;
begin
WClock <= iv[11];
RClock <= iv[10];
WR_EN  <= iv[9];
RD_EN <= iv[8];
Aclr  <= iv[7];
WR_addr <= iv [6:0];
end
endtask


task check_output;
begin
   if (|RD_ADDR_exp!==1'bx && RD_addr!==RD_ADDR_exp) begin
       $display("%t Error on RD_ADDR Expected=%h Actual=%h Index=%0d",
                $realtime,RD_ADDR_exp,RD_addr,index_o);
   end
end
endtask


initial 
begin

index_i = 1;
index_o = 1;
$display("Reading files %0d Vectors",vectors);
$readmemb("bvec.in",vector_i);
$readmemb("bvec.out",vector_o);
$readmemb("bvec.bi",vector_b);



fork: main
 for(index_i = 1; index_i <= vectors; index_i = index_i + 1) begin
  //$display("get into for loop 1");
  iv = vector_i[index_i];
  apply_iv;
  #half_cycle;
 end
 
 for (index_o = 1; index_o <= vectors; index_o = index_o + 1) begin
  //$display("get into for loop 2");
  ov = vector_o[index_o];
  #(half_cycle - strobe);
   check_output;
  #strobe;
 end


 for(index_b = 1; index_b <= vectors; index_b = index_b + 1) begin
   //$display("get into for loop 1");
   bi = vector_b[index_b];
   if (RD_EN == 0 && WR_EN == 1) 
      begin
      test <= 8'bzzzzzzzz;
      #half_cycle;
      end
   else  
    begin
    test <= bi; 
    #(half_cycle);
    end
 end

join

$display("Done with Test Vectors");
$stop;

end

//Instantiation of the product module.
top top_0(
	.SRAM_DATA(SRAM_DATA),
	.WR_addr(WR_addr),
	.RD_addr(RD_addr),
	.WR_EN(WR_EN),
	.RD_EN(RD_EN),
	.Aclr(Aclr),
	.WClock(WClock),
	.RClock(RClock)
	);



endmodule
